@@ -12,19 +12,22 @@ import pandas as pd
 import prophet
 
 from darts.logging import execute_and_suppress_output, get_logger, raise_if
-from darts.models.forecasting.forecasting_model import DualCovariatesForecastingModel
+from darts.models.forecasting.forecasting_model import (
+    FutureCovariatesLocalForecastingModel,
+)
 from darts.timeseries import TimeSeries
 
 logger = get_logger(__name__)
 logger.level = logging.WARNING  # set to warning to suppress prophet logs
 
 
-class Prophet(DualCovariatesForecastingModel):
+class Prophet(FutureCovariatesLocalForecastingModel):
     def __init__(
         self,
         add_seasonalities: Optional[Union[dict, List[dict]]] = None,
         country_holidays: Optional[str] = None,
         suppress_stdout_stderror: bool = True,
+        add_encoders: Optional[dict] = None,
         **prophet_kwargs,
     ):
         """Facebook Prophet
@@ -69,13 +72,33 @@ class Prophet(DualCovariatesForecastingModel):
             Egypt (EG), China (CN), and Russia (RU).
         suppress_stdout_stderror
             Optionally suppress the log output produced by Prophet during training.
+        add_encoders
+            A large number of future covariates can be automatically generated with `add_encoders`.
+            This can be done by adding multiple pre-defined index encoders and/or custom user-made functions that
+            will be used as index encoders. Additionally, a transformer such as Darts' :class:`Scaler` can be added to
+            transform the generated covariates. This happens all under one hood and only needs to be specified at
+            model creation.
+            Read :meth:`SequentialEncoder <darts.dataprocessing.encoders.SequentialEncoder>` to find out more about
+            ``add_encoders``. Default: ``None``. An example showing some of ``add_encoders`` features:
+
+            .. highlight:: python
+            .. code-block:: python
+
+                add_encoders={
+                    'cyclic': {'future': ['month']},
+                    'datetime_attribute': {'future': ['hour', 'dayofweek']},
+                    'position': {'future': ['relative']},
+                    'custom': {'future': [lambda idx: (idx.year - 1950) / 50]},
+                    'transformer': Scaler()
+                }
+            ..
         prophet_kwargs
             Some optional keyword arguments for Prophet.
             For information about the parameters see:
             `The Prophet source code <https://github.com/facebook/prophet/blob/master/python/prophet/forecaster.py>`_.
         """
 
-        super().__init__()
+        super().__init__(add_encoders=add_encoders)
 
         self._auto_seasonalities = self._extract_auto_seasonality(prophet_kwargs)
 
@@ -102,6 +125,7 @@ class Prophet(DualCovariatesForecastingModel):
     def _fit(self, series: TimeSeries, future_covariates: Optional[TimeSeries] = None):
 
         super()._fit(series, future_covariates)
+        self._assert_univariate(series)
         series = self.training_series
 
         fit_df = pd.DataFrame(
@@ -148,6 +172,7 @@ class Prophet(DualCovariatesForecastingModel):
         n: int,
         future_covariates: Optional[TimeSeries] = None,
         num_samples: int = 1,
+        verbose: bool = False,
     ) -> TimeSeries:
 
         super()._predict(n, future_covariates, num_samples)
