@@ -1,7 +1,7 @@
 """
 Regression Model
 ----------------
-A `RegressionModel` forecasts future values of a target series based on lagged values of
+A `RegressionModel` forecasts future values of a target series based on
 
 * The target series (past lags only)
 
@@ -9,6 +9,8 @@ A `RegressionModel` forecasts future values of a target series based on lagged v
 
 * An optional future_covariates series (possibly past and future lags)
 
+* Available static covariates
+
 
 The regression models are learned in a supervised way, and they can wrap around any "scikit-learn like" regression model
 acting on tabular data having ``fit()`` and ``predict()`` methods.
@@ -21,25 +23,35 @@ Behind the scenes this model is tabularizing the time series data to make it wor
 The lags can be specified either using an integer - in which case it represents the _number_ of (past or future) lags
 to take into consideration, or as a list - in which case the lags have to be enumerated (strictly negative values
 denoting past lags and positive values including 0 denoting future lags).
+When static covariates are present, they are appended to the lagged features. When multiple time series are passed,
+if their static covariates do not have the same size, the shorter ones are padded with 0 valued features.
 """
 
-import math
 from collections import OrderedDict
 from typing import List, Optional, Sequence, Tuple, Union
 
 import numpy as np
-from catboost import CatBoostRegressor
 from sklearn.linear_model import LinearRegression
 
 from darts.logging import get_logger, raise_if, raise_if_not, raise_log
 from darts.models.forecasting.forecasting_model import GlobalForecastingModel
 from darts.timeseries import TimeSeries
-from darts.utils.data.tabularization import _create_lagged_data
+from darts.utils.data.tabularization import _add_static_covariates, _create_lagged_data
 from darts.utils.multioutput import MultiOutputRegressor
 from darts.utils.utils import _check_quantiles, seq2series, series2seq
 
 logger = get_logger(__name__)
 
+try:
+    from catboost import CatBoostRegressor
+except ModuleNotFoundError:
+    logger.warning(
+        "The catboost module could not be imported. "
+        "To enable support for CatBoostRegressor, "
+        "follow the instruction in the README: "
+        "https://github.com/unit8co/darts/blob/master/INSTALL.md"
+    )
+
 
 class RegressionModel(GlobalForecastingModel):
     def __init__(
@@ -50,6 +62,7 @@ class RegressionModel(GlobalForecastingModel):
         output_chunk_length: int = 1,
         add_encoders: Optional[dict] = None,
         model=None,
+        multi_models: Optional[bool] = True,
     ):
         """Regression Model
         Can be used to fit any scikit-learn-like regressor class to predict the target time series from lagged values.
@@ -58,13 +71,13 @@ class RegressionModel(GlobalForecastingModel):
         ----------
         lags
             Lagged target values used to predict the next time step. If an integer is given the last `lags` past lags
-            are used (from -1 backward). Otherwise a list of integers with lags is required (each lag must be < 0).
+            are used (from -1 backward). Otherwise, a list of integers with lags is required (each lag must be < 0).
         lags_past_covariates
             Number of lagged past_covariates values used to predict the next time step. If an integer is given the last
             `lags_past_covariates` past lags are used (inclusive, starting from lag -1). Otherwise a list of integers
             with lags < 0 is required.
         lags_future_covariates
-            Number of lagged future_covariates values used to predict the next time step. If an tuple (past, future) is
+            Number of lagged future_covariates values used to predict the next time step. If a tuple (past, future) is
             given the last `past` lags in the past are used (inclusive, starting from lag -1) along with the first
             `future` future lags (starting from 0 - the prediction time - up to `future - 1` included). Otherwise a list
             of integers with lags is required.
@@ -78,7 +91,7 @@ class RegressionModel(GlobalForecastingModel):
             will be used as index encoders. Additionally, a transformer such as Darts' :class:`Scaler` can be added to
             transform the generated covariates. This happens all under one hood and only needs to be specified at
             model creation.
-            Read :meth:`SequentialEncoder <darts.utils.data.encoders.SequentialEncoder>` to find out more about
+            Read :meth:`SequentialEncoder <darts.dataprocessing.encoders.SequentialEncoder>` to find out more about
             ``add_encoders``. Default: ``None``. An example showing some of ``add_encoders`` features:
 
             .. highlight:: python
@@ -97,6 +110,10 @@ class RegressionModel(GlobalForecastingModel):
             support multi-output regression for multivariate timeseries, in which case one regressor
             will be used per component in the multivariate series.
             If None, defaults to: ``sklearn.linear_model.LinearRegression(n_jobs=-1)``.
+
+        multi_models
+            If True, a separate model will be trained for each future lag to predict. If False, a single model is
+            trained to predict at step 'output_chunk_length' in the future. Default: True.
         """
 
         super().__init__(add_encoders=add_encoders)
@@ -105,6 +122,7 @@ class RegressionModel(GlobalForecastingModel):
         self.lags = {}
         self.output_chunk_length = None
         self.input_dim = None
+        self.multi_models = multi_models
 
         # model checks
         if self.model is None:
@@ -221,45 +239,55 @@ class RegressionModel(GlobalForecastingModel):
         )
         self.output_chunk_length = output_chunk_length
 
+        self.pred_dim = self.output_chunk_length if self.multi_models else 1
+
     @property
-    def _model_encoder_settings(self) -> Tuple[int, int, bool, bool]:
-        lags_covariates = {
-            lag for key in ["past", "future"] for lag in self.lags.get(key, [])
-        }
-        if lags_covariates:
-            # for lags < 0 we need to take `n` steps backwards from past and/or historic future covariates
-            # for minimum lag = -1 -> steps_back_inclusive = 1
-            # inclusive means n steps back including the end of the target series
-            n_steps_back_inclusive = abs(min(min(lags_covariates), 0))
-            # for lags >= 0 we need to take `n` steps ahead from future covariates
-            # for maximum lag = 0 -> output_chunk_length = 1
-            # exclusive means n steps ahead after the last step of the target series
-            n_steps_ahead_exclusive = max(max(lags_covariates), 0) + 1
-            takes_past_covariates = "past" in self.lags
-            takes_future_covariates = "future" in self.lags
-        else:
-            n_steps_back_inclusive = 0
-            n_steps_ahead_exclusive = 0
-            takes_past_covariates = False
-            takes_future_covariates = False
+    def _model_encoder_settings(
+        self,
+    ) -> Tuple[int, int, bool, bool, Optional[List[int]], Optional[List[int]]]:
+        target_lags = self.lags.get("target", [0])
+        lags_past_covariates = self.lags.get("past", None)
+        if lags_past_covariates is not None:
+            lags_past_covariates = [
+                min(lags_past_covariates)
+                - int(not self.multi_models) * (self.output_chunk_length - 1),
+                max(lags_past_covariates),
+            ]
+        lags_future_covariates = self.lags.get("future", None)
+        if lags_future_covariates is not None:
+            lags_future_covariates = [
+                min(lags_future_covariates)
+                - int(not self.multi_models) * (self.output_chunk_length - 1),
+                max(lags_future_covariates),
+            ]
         return (
-            n_steps_back_inclusive,
-            n_steps_ahead_exclusive,
-            takes_past_covariates,
-            takes_future_covariates,
+            abs(min(target_lags)),
+            self.output_chunk_length,
+            lags_past_covariates is not None,
+            lags_future_covariates is not None,
+            lags_past_covariates,
+            lags_future_covariates,
         )
 
-    def _get_encoders_n(self, n) -> int:
-        """Returns the `n` encoder prediction steps specific to RegressionModels.
-        This will generate slightly more past covariates than the minimum requirement when using past and future
-        covariate lags simultaneously. This is because encoders were written for TorchForecastingModels where we only
-        needed `n` future covariates. For RegressionModel we need `n + max_future_lag`
-        """
-        _, n_steps_ahead, _, takes_future_covariates = self._model_encoder_settings
-        if not takes_future_covariates:
-            return n
-        else:
-            return n + (n_steps_ahead - 1)
+    @property
+    def extreme_lags(self):
+        min_target_lag = self.lags.get("target")[0] if "target" in self.lags else None
+        max_target_lag = self.output_chunk_length
+        min_past_cov_lag = self.lags.get("past")[0] if "past" in self.lags else None
+        min_future_cov_lag = (
+            self.lags.get("future")[0] if "future" in self.lags else None
+        )
+        max_future_cov_lag = (
+            self.lags.get("future")[-1] if "future" in self.lags else None
+        )
+
+        return (
+            min_target_lag,
+            max_target_lag,
+            min_past_cov_lag,
+            min_future_cov_lag,
+            max_future_cov_lag,
+        )
 
     @property
     def min_train_series_length(self) -> int:
@@ -305,6 +333,11 @@ class RegressionModel(GlobalForecastingModel):
             lags_past_covariates=lags_past_covariates,
             lags_future_covariates=lags_future_covariates,
             max_samples_per_ts=max_samples_per_ts,
+            multi_models=self.multi_models,
+        )
+
+        training_samples = _add_static_covariates(
+            self.model, target_series, training_samples
         )
 
         return training_samples, training_labels
@@ -377,6 +410,11 @@ class RegressionModel(GlobalForecastingModel):
                 future_covariates=future_covariates,
             )
 
+        if past_covariates is not None:
+            self._uses_past_covariates = True
+        if future_covariates is not None:
+            self._uses_future_covariates = True
+
         for covs, name in zip([past_covariates, future_covariates], ["past", "future"]):
             raise_if(
                 covs is not None and name not in self.lags,
@@ -398,7 +436,9 @@ class RegressionModel(GlobalForecastingModel):
         }
 
         # if multi-output regression
-        if not series[0].is_univariate or self.output_chunk_length > 1:
+        if not series[0].is_univariate or (
+            self.output_chunk_length > 1 and self.multi_models
+        ):
             # and model isn't wrapped already
             if not isinstance(self.model, MultiOutputRegressor):
                 # check whether model supports multi-output regression natively
@@ -446,6 +486,7 @@ class RegressionModel(GlobalForecastingModel):
         past_covariates: Optional[Union[TimeSeries, Sequence[TimeSeries]]] = None,
         future_covariates: Optional[Union[TimeSeries, Sequence[TimeSeries]]] = None,
         num_samples: int = 1,
+        verbose: bool = False,
         **kwargs,
     ) -> Union[TimeSeries, Sequence[TimeSeries]]:
         """Forecasts values for `n` time steps after the end of the series.
@@ -522,63 +563,70 @@ class RegressionModel(GlobalForecastingModel):
         )
 
         # prediction preprocessing
-
         covariates = {
             "past": (past_covariates, self.lags.get("past")),
             "future": (future_covariates, self.lags.get("future")),
         }
 
+        # prepare one_shot shift and step
+        if self.multi_models:
+            shift = 0
+            step = self.output_chunk_length
+        else:
+            shift = self.output_chunk_length - 1
+            step = 1
+
         # dictionary containing covariate data over time span required for prediction
         covariate_matrices = {}
         # dictionary containing covariate lags relative to minimum covariate lag
         relative_cov_lags = {}
-        # number of prediction steps given forecast horizon and output_chunk_length
-        n_pred_steps = math.ceil(n / self.output_chunk_length)
         for cov_type, (covs, lags) in covariates.items():
-            if covs is not None:
-                relative_cov_lags[cov_type] = np.array(lags) - lags[0]
-                covariate_matrices[cov_type] = []
-                for idx, (ts, cov) in enumerate(zip(series, covs)):
-                    # calculating first and last prediction time steps
-                    first_pred_ts = ts.end_time() + 1 * ts.freq
-                    last_pred_ts = (
-                        first_pred_ts
-                        + ((n_pred_steps - 1) * self.output_chunk_length) * ts.freq
-                    )
-                    # calculating first and last required time steps
-                    first_req_ts = first_pred_ts + lags[0] * ts.freq
-                    last_req_ts = last_pred_ts + lags[-1] * ts.freq
+            if covs is None:
+                continue
 
-                    # check for sufficient covariate data
-                    raise_if_not(
-                        cov.start_time() <= first_req_ts
-                        and cov.end_time() >= last_req_ts,
-                        f"The corresponding {cov_type}_covariate of the series at index {idx} isn't sufficiently long. "
-                        f"Given horizon `n={n}`, `min(lags_{cov_type}_covariates)={lags[0]}`, "
-                        f"`max(lags_{cov_type}_covariates)={lags[-1]}` and "
-                        f"`output_chunk_length={self.output_chunk_length}`\n"
-                        f"the {cov_type}_covariate has to range from {first_req_ts} until {last_req_ts} (inclusive), "
-                        f"but it ranges only from {cov.start_time()} until {cov.end_time()}.",
+            relative_cov_lags[cov_type] = np.array(lags) - lags[0]
+            covariate_matrices[cov_type] = []
+            for idx, (ts, cov) in enumerate(zip(series, covs)):
+                # how many steps to go back from end of target series for start of covariates
+                steps_back = -(min(lags) + 1) + shift
+                lags_diff = max(lags) - min(lags) + 1
+                # over how many steps the covariates range
+                n_steps = lags_diff + max(0, n - self.output_chunk_length) + shift
+
+                # calculate first and last required covariate time steps
+                start_ts = ts.end_time() - ts.freq * steps_back
+                end_ts = start_ts + ts.freq * (n_steps - 1)
+
+                # check for sufficient covariate data
+                if not (cov.start_time() <= start_ts and cov.end_time() >= end_ts):
+                    raise_log(
+                        ValueError(
+                            f"The corresponding {cov_type}_covariate of the series at index {idx} isn't sufficiently "
+                            f"long. Given horizon `n={n}`, `min(lags_{cov_type}_covariates)={lags[0]}`, "
+                            f"`max(lags_{cov_type}_covariates)={lags[-1]}` and "
+                            f"`output_chunk_length={self.output_chunk_length}`, the {cov_type}_covariate has to range "
+                            f"from {start_ts} until {end_ts} (inclusive), but it ranges only from {cov.start_time()} "
+                            f"until {cov.end_time()}."
+                        ),
+                        logger=logger,
                     )
 
-                    # Note: we use slice() rather than the [] operator because
-                    # for integer-indexed series [] does not act on the time index.
-                    last_req_ts = (
-                        # For range indexes, we need to make the end timestamp inclusive here
-                        last_req_ts + ts.freq
-                        if ts.has_range_index
-                        else last_req_ts
-                    )
-                    covariate_matrices[cov_type].append(
-                        cov.slice(first_req_ts, last_req_ts).values(copy=False)
-                    )
+                # use slice() instead of [] as for integer-indexed series [] does not act on time index
+                # for range indexes, we make the end timestamp inclusive here
+                end_ts = end_ts + ts.freq if ts.has_range_index else end_ts
+                covariate_matrices[cov_type].append(
+                    cov.slice(start_ts, end_ts).values(copy=False)
+                )
 
-                covariate_matrices[cov_type] = np.stack(covariate_matrices[cov_type])
+            covariate_matrices[cov_type] = np.stack(covariate_matrices[cov_type])
 
         series_matrix = None
         if "target" in self.lags:
             series_matrix = np.stack(
-                [ts[self.lags["target"][0] :].values(copy=False) for ts in series]
+                [
+                    ts.values(copy=False)[self.lags["target"][0] - shift :, :]
+                    for ts in series
+                ]
             )
 
         # repeat series_matrix to shape (num_samples * num_series, n_lags, n_components)
@@ -590,21 +638,31 @@ class RegressionModel(GlobalForecastingModel):
             covariate_matrices[cov_type] = np.repeat(data, num_samples, axis=0)
         # prediction
         predictions = []
+        last_step_shift = 0
+
         # t_pred indicates the number of time steps after the first prediction
-        for t_pred in range(0, n, self.output_chunk_length):
+        for t_pred in range(0, n, step):
+            # in case of autoregressive forecast `(t_pred > 0)` and if `n` is not a round multiple of `step`,
+            # we have to step back `step` from `n` in the last iteration
+            if 0 < n - t_pred < step and t_pred > 0:
+                last_step_shift = t_pred - (n - step)
+                t_pred = n - step
+
             np_X = []
             # retrieve target lags
             if "target" in self.lags:
-
-                target_matrix = (
-                    np.concatenate([series_matrix, *predictions], axis=1)
-                    if predictions
-                    else series_matrix
-                )
-                np_X.append(
-                    target_matrix[:, self.lags["target"]].reshape(
-                        len(series) * num_samples, -1
+                if predictions:
+                    series_matrix = np.concatenate(
+                        [series_matrix, predictions[-1]], axis=1
                     )
+                np_X.append(
+                    series_matrix[
+                        :,
+                        [
+                            lag - (shift + last_step_shift)
+                            for lag in self.lags["target"]
+                        ],
+                    ].reshape(len(series) * num_samples, -1)
                 )
             # retrieve covariate lags, enforce order (dict only preserves insertion order for python 3.6+)
             for cov_type in ["past", "future"]:
@@ -617,11 +675,13 @@ class RegressionModel(GlobalForecastingModel):
 
             # concatenate retrieved lags
             X = np.concatenate(np_X, axis=1)
+            X = _add_static_covariates(self.model, series, X)
+
             # X has shape (n_series * n_samples, n_regression_features)
             prediction = self._predict_and_sample(X, num_samples, **kwargs)
             # prediction shape (n_series * n_samples, output_chunk_length, n_components)
             # append prediction to final predictions
-            predictions.append(prediction)
+            predictions.append(prediction[:, last_step_shift:])
 
         # concatenate and use first n points as prediction
         predictions = np.concatenate(predictions, axis=1)[:, :n]
@@ -643,11 +703,16 @@ class RegressionModel(GlobalForecastingModel):
     ) -> np.ndarray:
         prediction = self.model.predict(x, **kwargs)
         k = x.shape[0]
-        return prediction.reshape(k, self.output_chunk_length, -1)
+
+        return prediction.reshape(k, self.pred_dim, -1)
 
     def __str__(self):
         return self.model.__str__()
 
+    @staticmethod
+    def _supports_static_covariates() -> bool:
+        return True
+
 
 class _LikelihoodMixin:
     """
@@ -694,18 +759,17 @@ class _LikelihoodMixin:
         X is of shape (n_series * n_samples, n_regression_features)
         """
         k = x.shape[0]
+
         if num_samples == 1:
             # return median
             fitted = self._model_container[0.5]
-            return fitted.predict(x, **kwargs).reshape(k, self.output_chunk_length, -1)
+            return fitted.predict(x, **kwargs).reshape(k, self.pred_dim, -1)
 
         model_outputs = []
         for quantile, fitted in self._model_container.items():
             self.model = fitted
             # model output has shape (n_series * n_samples, output_chunk_length, n_components)
-            model_output = fitted.predict(x, **kwargs).reshape(
-                k, self.output_chunk_length, -1
-            )
+            model_output = fitted.predict(x, **kwargs).reshape(k, self.pred_dim, -1)
             model_outputs.append(model_output)
         model_outputs = np.stack(model_outputs, axis=-1)
         # model_outputs has shape (n_series * n_samples, output_chunk_length, n_components, n_quantiles)
@@ -738,7 +802,7 @@ class _LikelihoodMixin:
             else:
                 output_slice = model_output[0, :, :]
 
-            return output_slice.reshape(k, self.output_chunk_length, -1)
+            return output_slice.reshape(k, self.pred_dim, -1)
 
         # probabilistic case
         # univariate & single-chunk output
@@ -759,7 +823,7 @@ class _LikelihoodMixin:
         where the last 2 dimensions are mu and sigma.
         """
         shape = model_output.shape
-        chunk_len = self.output_chunk_length
+        chunk_len = self.pred_dim
 
         # treating each component separately
         mu_sigma_list = [model_output[i, :, :] for i in range(shape[0])]
@@ -783,9 +847,7 @@ class _LikelihoodMixin:
         """
         k = x.shape[0]
 
-        model_output = self.model.predict(x, **kwargs).reshape(
-            k, self.output_chunk_length, -1
-        )
+        model_output = self.model.predict(x, **kwargs).reshape(k, self.pred_dim, -1)
         if num_samples == 1:
             return model_output
 
