@@ -450,6 +450,7 @@ class _TFTModule(PLMixedCovariatesModule):
         """
         x_cont_past, x_cont_future, x_static = x_in
         dim_samples, dim_time, dim_variable = 0, 1, 2
+        device = x_in[0].device
 
         batch_size = x_cont_past.shape[dim_samples]
         encoder_length = self.input_chunk_length
@@ -463,21 +464,21 @@ class _TFTModule(PLMixedCovariatesModule):
                     time_steps=time_steps,
                     batch_size=batch_size,
                     dtype=x_cont_past.dtype,
-                    device=self.device,
+                    device=device,
                 )
             else:
                 self.attention_mask = self.get_attention_mask_future(
                     encoder_length=encoder_length,
                     decoder_length=decoder_length,
                     batch_size=batch_size,
-                    device=self.device,
+                    device=device,
                 )
             if self.add_relative_index:
                 self.relative_index = self.get_relative_index(
                     encoder_length=encoder_length,
                     decoder_length=decoder_length,
                     batch_size=batch_size,
-                    device=self.device,
+                    device=device,
                     dtype=x_cont_past.dtype,
                 )
 
@@ -541,7 +542,7 @@ class _TFTModule(PLMixedCovariatesModule):
             static_embedding = torch.zeros(
                 (x_cont_past.shape[0], self.hidden_size),
                 dtype=x_cont_past.dtype,
-                device=self.device,
+                device=device,
             )
 
         static_context_expanded = self.expand_static_context(
@@ -790,7 +791,7 @@ class TFTModel(MixedCovariatesTorchModel):
             will be used as index encoders. Additionally, a transformer such as Darts' :class:`Scaler` can be added to
             transform the generated covariates. This happens all under one hood and only needs to be specified at
             model creation.
-            Read :meth:`SequentialEncoder <darts.utils.data.encoders.SequentialEncoder>` to find out more about
+            Read :meth:`SequentialEncoder <darts.dataprocessing.encoders.SequentialEncoder>` to find out more about
             ``add_encoders``. Default: ``None``. An example showing some of ``add_encoders`` features:
 
             .. highlight:: python
@@ -900,6 +901,7 @@ class TFTModel(MixedCovariatesTorchModel):
             each tensor has shape (n_timesteps, n_variables)
             - past/historic tensors have shape (input_chunk_length, n_variables)
             - future tensors have shape (output_chunk_length, n_variables)
+            - static covariates have shape (component, static variable)
 
         Darts Interpretation of pytorch-forecasting's TimeSeriesDataSet:
             time_varying_knowns : future_covariates (including historic_future_covariates)
