@@ -117,6 +117,16 @@ class BacktestingTestCase(DartsBaseTestClass):
         )
         self.assertEqual(score, 1.0)
 
+        # using several metric function should not affect the backtest
+        score = NaiveDrift().backtest(
+            linear_series,
+            train_length=10000,
+            start=pd.Timestamp("20000201"),
+            forecast_horizon=3,
+            metric=[r2_score, mape],
+        )
+        np.testing.assert_almost_equal(score, np.array([1.0, 0.0]))
+
         # window of size 2 is too small for naive drift
         with self.assertRaises(ValueError):
             NaiveDrift().backtest(
@@ -188,8 +198,8 @@ class BacktestingTestCase(DartsBaseTestClass):
             )
 
         # univariate model + multivariate series
-        with self.assertRaises(AssertionError):
-            NaiveDrift().backtest(
+        with self.assertRaises(ValueError):
+            FFT().backtest(
                 linear_series_multi, start=pd.Timestamp("20000201"), forecast_horizon=3
             )
 
@@ -311,7 +321,7 @@ class BacktestingTestCase(DartsBaseTestClass):
 
         # Using RandomForest's start default value
         score = RandomForest(lags=12, random_state=0).backtest(
-            series=target, forecast_horizon=3, metric=r2_score
+            series=target, forecast_horizon=3, start=0.5, metric=r2_score
         )
         self.assertGreater(score, 0.95)
 
