@@ -200,6 +200,26 @@ class TimeSeriesTestCase(DartsBaseTestClass):
                 (abs(q_ts.values() - np.quantile(values, q=q, axis=2)) < 1e-3).all()
             )
 
+    def test_quantiles_df(self):
+        q = (0.01, 0.1, 0.5, 0.95)
+        values = np.random.rand(10, 1, 1000)
+        ar = xr.DataArray(
+            values,
+            dims=("time", "component", "sample"),
+            coords={"time": self.times, "component": ["a"]},
+        )
+        ts = TimeSeries(ar)
+        q_ts = ts.quantiles_df(q)
+        for col in q_ts:
+            q = float(str(col).replace("a_", ""))
+            self.assertTrue(
+                abs(
+                    q_ts[col].to_numpy().reshape(10, 1)
+                    - np.quantile(values, q=q, axis=2)
+                    < 1e-3
+                ).all()
+            )
+
     def test_alt_creation(self):
         with self.assertRaises(ValueError):
             # Series cannot be lower than three without passing frequency as argument to constructor,
@@ -377,6 +397,18 @@ class TimeSeriesTestCase(DartsBaseTestClass):
             with test_case.assertRaises(ValueError):
                 test_series.split_before(value)
 
+        # Test split points between series indeces
+        times = pd.date_range("20130101", "20130120", freq="2D")
+        pd_series = pd.Series(range(10), index=times)
+        test_series2: TimeSeries = TimeSeries.from_series(pd_series)
+        split_date = pd.Timestamp("20130110")
+        seriesM, seriesN = test_series2.split_before(split_date)
+        seriesO, seriesP = test_series2.split_after(split_date)
+        test_case.assertLess(seriesM.end_time(), split_date)
+        test_case.assertGreaterEqual(seriesN.start_time(), split_date)
+        test_case.assertLessEqual(seriesO.end_time(), split_date)
+        test_case.assertGreater(seriesP.start_time(), split_date)
+
     @staticmethod
     def helper_test_drop(test_case, test_series: TimeSeries):
         seriesA = test_series.drop_after(pd.Timestamp("20130105"))
@@ -496,19 +528,82 @@ class TimeSeriesTestCase(DartsBaseTestClass):
         seriesA, seriesB = test_series.split_after(pd.Timestamp("20130106"))
         test_case.assertEqual(seriesA.append(seriesB), test_series)
         test_case.assertEqual(seriesA.append(seriesB).freq, test_series.freq)
+        test_case.assertTrue(
+            test_series.time_index.equals(seriesA.append(seriesB).time_index)
+        )
 
         # Creating a gap is not allowed
         seriesC = test_series.drop_before(pd.Timestamp("20130108"))
         with test_case.assertRaises(ValueError):
             seriesA.append(seriesC)
 
-        # Changing frequence is not allowed
+        # Changing frequency is not allowed
         seriesM = TimeSeries.from_times_and_values(
             pd.date_range("20130107", "20130507", freq="30D"), range(5)
         )
         with test_case.assertRaises(ValueError):
             seriesA.append(seriesM)
 
+    @staticmethod
+    def helper_test_append_values(test_case, test_series: TimeSeries):
+        # reconstruct series
+        seriesA, seriesB = test_series.split_after(pd.Timestamp("20130106"))
+        arrayB = seriesB.all_values()
+        test_case.assertEqual(seriesA.append_values(arrayB), test_series)
+        test_case.assertTrue(
+            test_series.time_index.equals(seriesA.append_values(arrayB).time_index)
+        )
+
+        # arrayB shape shouldn't affect append_values output:
+        squeezed_arrayB = arrayB.squeeze()
+        test_case.assertEqual(seriesA.append_values(squeezed_arrayB), test_series)
+        test_case.assertTrue(
+            test_series.time_index.equals(
+                seriesA.append_values(squeezed_arrayB).time_index
+            )
+        )
+
+    @staticmethod
+    def helper_test_prepend(test_case, test_series: TimeSeries):
+        # reconstruct series
+        seriesA, seriesB = test_series.split_after(pd.Timestamp("20130106"))
+        test_case.assertEqual(seriesB.prepend(seriesA), test_series)
+        test_case.assertEqual(seriesB.prepend(seriesA).freq, test_series.freq)
+        test_case.assertTrue(
+            test_series.time_index.equals(seriesB.prepend(seriesA).time_index)
+        )
+
+        # Creating a gap is not allowed
+        seriesC = test_series.drop_before(pd.Timestamp("20130108"))
+        with test_case.assertRaises(ValueError):
+            seriesC.prepend(seriesA)
+
+        # Changing frequency is not allowed
+        seriesM = TimeSeries.from_times_and_values(
+            pd.date_range("20130107", "20130507", freq="30D"), range(5)
+        )
+        with test_case.assertRaises(ValueError):
+            seriesM.prepend(seriesA)
+
+    @staticmethod
+    def helper_test_prepend_values(test_case, test_series: TimeSeries):
+        # reconstruct series
+        seriesA, seriesB = test_series.split_after(pd.Timestamp("20130106"))
+        arrayA = seriesA.data_array().values
+        test_case.assertEqual(seriesB.prepend_values(arrayA), test_series)
+        test_case.assertTrue(
+            test_series.time_index.equals(seriesB.prepend_values(arrayA).time_index)
+        )
+
+        # arrayB shape shouldn't affect append_values output:
+        squeezed_arrayA = arrayA.squeeze()
+        test_case.assertEqual(seriesB.prepend_values(squeezed_arrayA), test_series)
+        test_case.assertTrue(
+            test_series.time_index.equals(
+                seriesB.prepend_values(squeezed_arrayA).time_index
+            )
+        )
+
     def test_slice(self):
         TimeSeriesTestCase.helper_test_slice(self, self.series1)
 
@@ -526,6 +621,53 @@ class TimeSeriesTestCase(DartsBaseTestClass):
 
     def test_append(self):
         TimeSeriesTestCase.helper_test_append(self, self.series1)
+        # Check `append` deals with `RangeIndex` series correctly:
+        series_1 = linear_timeseries(start=1, length=5, freq=2)
+        series_2 = linear_timeseries(start=11, length=2, freq=2)
+        appended = series_1.append(series_2)
+        expected_vals = np.concatenate(
+            [series_1.all_values(), series_2.all_values()], axis=0
+        )
+        expected_idx = pd.RangeIndex(start=1, stop=15, step=2)
+        self.assertTrue(np.allclose(appended.all_values(), expected_vals))
+        self.assertTrue(appended.time_index.equals(expected_idx))
+
+    def test_append_values(self):
+        TimeSeriesTestCase.helper_test_append_values(self, self.series1)
+        # Check `append_values` deals with `RangeIndex` series correctly:
+        series = linear_timeseries(start=1, length=5, freq=2)
+        appended = series.append_values(np.ones((2, 1, 1)))
+        expected_vals = np.concatenate(
+            [series.all_values(), np.ones((2, 1, 1))], axis=0
+        )
+        expected_idx = pd.RangeIndex(start=1, stop=15, step=2)
+        self.assertTrue(np.allclose(appended.all_values(), expected_vals))
+        self.assertTrue(appended.time_index.equals(expected_idx))
+
+    def test_prepend(self):
+        TimeSeriesTestCase.helper_test_prepend(self, self.series1)
+        # Check `prepend` deals with `RangeIndex` series correctly:
+        series_1 = linear_timeseries(start=1, length=5, freq=2)
+        series_2 = linear_timeseries(start=11, length=2, freq=2)
+        prepended = series_2.prepend(series_1)
+        expected_vals = np.concatenate(
+            [series_1.all_values(), series_2.all_values()], axis=0
+        )
+        expected_idx = pd.RangeIndex(start=1, stop=15, step=2)
+        self.assertTrue(np.allclose(prepended.all_values(), expected_vals))
+        self.assertTrue(prepended.time_index.equals(expected_idx))
+
+    def test_prepend_values(self):
+        TimeSeriesTestCase.helper_test_prepend_values(self, self.series1)
+        # Check `prepend_values` deals with `RangeIndex` series correctly:
+        series = linear_timeseries(start=1, length=5, freq=2)
+        prepended = series.prepend_values(np.ones((2, 1, 1)))
+        expected_vals = np.concatenate(
+            [np.ones((2, 1, 1)), series.all_values()], axis=0
+        )
+        expected_idx = pd.RangeIndex(start=-3, stop=11, step=2)
+        self.assertTrue(np.allclose(prepended.all_values(), expected_vals))
+        self.assertTrue(prepended.time_index.equals(expected_idx))
 
     def test_with_values(self):
         vals = np.random.rand(5, 10, 3)
@@ -839,6 +981,17 @@ class TimeSeriesTestCase(DartsBaseTestClass):
             resampled_timeseries.pd_series().at[pd.Timestamp("20130109")], 8
         )
 
+        # using loffset to avoid nan in the first value
+        times = pd.date_range(
+            start=pd.Timestamp("20200101233000"), periods=10, freq="15T"
+        )
+        pd_series = pd.Series(range(10), index=times)
+        timeseries = TimeSeries.from_series(pd_series)
+        resampled_timeseries = timeseries.resample(freq="1h", loffset="30T")
+        self.assertEqual(
+            resampled_timeseries.pd_series().at[pd.Timestamp("20200101233000")], 0
+        )
+
     def test_short_series_creation(self):
         # test missing freq argument error when filling missing dates on short time series
         with self.assertRaises(ValueError):
@@ -1035,6 +1188,10 @@ class TimeSeriesTestCase(DartsBaseTestClass):
             [1] * 10 + 1 * [np.nan] + [1] * 13 + 5 * [np.nan] + [1] * 18 + 9 * [np.nan],
             index=times4,
         )
+        pd_series7 = pd.Series(
+            [1] * 10 + 1 * [0] + [1] * 13 + 5 * [2] + [1] * 18 + 9 * [6],
+            index=times4,
+        )
 
         series1 = TimeSeries.from_series(pd_series1)
         series2 = TimeSeries.from_series(pd_series2)
@@ -1042,6 +1199,7 @@ class TimeSeriesTestCase(DartsBaseTestClass):
         series4 = TimeSeries.from_series(pd_series4)
         series5 = TimeSeries.from_series(pd_series5)
         series6 = TimeSeries.from_series(pd_series6)
+        series7 = TimeSeries.from_series(pd_series7)
 
         gaps1 = series1.gaps()
         self.assertTrue(
@@ -1111,6 +1269,8 @@ class TimeSeriesTestCase(DartsBaseTestClass):
                 )
             ).all()
         )
+        gaps7 = series7.gaps()
+        self.assertTrue(gaps7.empty)
 
         # test gaps detection on integer-indexed series
         values = np.array([1, 2, np.nan, np.nan, 3, 4, np.nan, 6])
@@ -1118,6 +1278,11 @@ class TimeSeriesTestCase(DartsBaseTestClass):
         ts = TimeSeries.from_times_and_values(times, values)
         np.testing.assert_equal(ts.gaps().values, np.array([[2, 3, 2], [6, 6, 1]]))
 
+        values = np.array([1, 2, 7, 8, 3, 4, 0, 6])
+        times = pd.RangeIndex(8)
+        ts = TimeSeries.from_times_and_values(times, values)
+        self.assertTrue(ts.gaps().empty)
+
     def test_longest_contiguous_slice(self):
         times = pd.date_range("20130101", "20130111")
         pd_series1 = pd.Series(
@@ -1761,6 +1926,55 @@ class TimeSeriesFromDataFrameTestCase(DartsBaseTestClass):
         self.assertEqual(ts.time_index.dtype, "datetime64[ns]")
         self.assertEqual(ts.time_index.name, "Time")
 
+    def test_time_col_with_tz(self):
+        # numpy and xarray don't support "timezone aware" pd.DatetimeIndex
+        # the BUGFIX removes timezone information without conversion
+
+        time_range_MS = pd.date_range(
+            start="20180501", end="20200301", freq="MS", tz="CET"
+        )
+        values = np.random.uniform(low=-10, high=10, size=len(time_range_MS))
+        # pd.DataFrame loses the tz information unless it is contained in its index
+        # (other columns are silently converted to UTC, with tz attribute set to None)
+        df = pd.DataFrame(data=values, index=time_range_MS)
+        ts = TimeSeries.from_dataframe(df=df)
+        self.assertEqual(list(ts.time_index), list(time_range_MS.tz_localize(None)))
+        self.assertEqual(list(ts.time_index.tz_localize("CET")), list(time_range_MS))
+        self.assertEqual(ts.time_index.tz, None)
+
+        serie = pd.Series(data=values, index=time_range_MS)
+        ts = TimeSeries.from_series(pd_series=serie)
+        self.assertEqual(list(ts.time_index), list(time_range_MS.tz_localize(None)))
+        self.assertEqual(list(ts.time_index.tz_localize("CET")), list(time_range_MS))
+        self.assertEqual(ts.time_index.tz, None)
+
+        ts = TimeSeries.from_times_and_values(times=time_range_MS, values=values)
+        self.assertEqual(list(ts.time_index), list(time_range_MS.tz_localize(None)))
+        self.assertEqual(list(ts.time_index.tz_localize("CET")), list(time_range_MS))
+        self.assertEqual(ts.time_index.tz, None)
+
+        time_range_H = pd.date_range(
+            start="20200518", end="20200521", freq="H", tz="CET"
+        )
+        values = np.random.uniform(low=-10, high=10, size=len(time_range_H))
+
+        df = pd.DataFrame(data=values, index=time_range_H)
+        ts = TimeSeries.from_dataframe(df=df)
+        self.assertEqual(list(ts.time_index), list(time_range_H.tz_localize(None)))
+        self.assertEqual(list(ts.time_index.tz_localize("CET")), list(time_range_H))
+        self.assertEqual(ts.time_index.tz, None)
+
+        serie = pd.Series(data=values, index=time_range_H)
+        ts = TimeSeries.from_series(pd_series=serie)
+        self.assertEqual(list(ts.time_index), list(time_range_H.tz_localize(None)))
+        self.assertEqual(list(ts.time_index.tz_localize("CET")), list(time_range_H))
+        self.assertEqual(ts.time_index.tz, None)
+
+        ts = TimeSeries.from_times_and_values(times=time_range_H, values=values)
+        self.assertEqual(list(ts.time_index), list(time_range_H.tz_localize(None)))
+        self.assertEqual(list(ts.time_index.tz_localize("CET")), list(time_range_H))
+        self.assertEqual(ts.time_index.tz, None)
+
     def test_time_col_convert_garbage(self):
         expected = [
             "2312312asdfdw",
